#include <stdio.h>
#include <math.h>
#include "siteinfo.h"
#include "syncprocess.h"
#include "globals.h"

using namespace std;

SiteInfo::SiteInfo() { 
	defaultPort = RSYNC_PORT;
	reset();
}

SiteInfo::SiteInfo(const string &d) {
	defaultPort = RSYNC_PORT;
	setDescription(d);
	initInfo();
}

SiteInfo::~SiteInfo() { /* Nothing to do here.. Qt does it all? */ }

void SiteInfo::createCommand(QProcess *rsyncProcess, RSyncStuff::RSyncOperation op, QString passFilename) {
	string source = "";
	string dest = "";
	rsyncProcess->clearArguments();
	rsyncProcess->addArgument(getRsyncBin().c_str());
	if (RSyncStuff::DoDownload == op)
		rsyncProcess->addArgument("-aPvvv"); // make dynamic later
	else if (RSyncStuff::DoUpload == op)
	 	rsyncProcess->addArgument("-aPvvv");
	else
		rsyncProcess->addArgument("-vvv"); // make dynamic later
	/* add a password file regardless of whethere a password is specified.
		 this prevents rsync from taking stalling while waiting for password input
		 when a passworded share is accessed without specifying a password
		if (getPass().length()) */
	rsyncProcess->addArgument("--password-file=" + passFilename);
	
	// rsyncProcess->addArgument("--bwlimit=16");

	if (!getLocalhost()) {
		if (RSyncStuff::DoDownload == op) {
			if (getUser().length())
				source = getUser() + "@";
			source += getHost() + "::" + getRemote();
			if ('/' != source[source.length() - 1])
				source += "/";
		 	source += getRemoteItem();
			dest = getLocalDir();
		}
		else if (RSyncStuff::DoUpload == op) {
			source = getLocalDir();
			if ('/' != source[source.length() - 1])
				source += "/";
			source += getLocalItem();
			if (getUser().length())
				dest = getUser() + "@";
			dest += getHost() + "::" + getRemote();
			// get rid of all these lame trailing slash checks at some point...
			if ('/' != dest[dest.length () - 1])
				dest += "/";
		}
		else { // listing only
			if (getUser().length())
				source = getUser() + "@";
			source += getHost() + "::" + getRemote();
		}
	}
	// find some way of listingOnly when using localhost
	// do this later, who cares now... localhost disabled
	else {
		//source = getSource();
		//dest = getDest();
	}

	rsyncProcess->addArgument(source.c_str());
	if (dest.length())
		rsyncProcess->addArgument(dest.c_str());
}

void SiteInfo::setDescription(const string &d) { description = d; }
string SiteInfo::getDescription() const { return description; }
void SiteInfo::setHost(const string &h) {
	if (h != LOCALHOST_T)
		host = h;
}
string SiteInfo::getHost() const { return host; }
void SiteInfo::setRemote(const string &r) { remote = r; }
string SiteInfo::getRemote() const { return remote; }
void SiteInfo::setLocalDir(const string &l) { localDir = l; } 
string SiteInfo::getLocalDir() const { return localDir; }
void SiteInfo::setRemoteItem(const string &r) { remoteItem = r; }
string SiteInfo::getRemoteItem() const { return remoteItem; }
void SiteInfo::setLocalItem(const string &l) { localItem = l; }
string SiteInfo::getLocalItem() const { return localItem; }
void SiteInfo::setUser(const string &u) { user = u; }
string SiteInfo::getUser() const { return user; }
void SiteInfo::setPass(const string &p) { password = p; }
string SiteInfo::getPass() const { return password; }
void SiteInfo::setRootPass(const string &p) { rootPass = p; } 
string SiteInfo::getRootPass() const { return rootPass; }
void SiteInfo::setComment(const string &c) { comment = c; } 
string SiteInfo::getComment() const { return comment; }
void SiteInfo::setRsyncBin(const string &r) { rsyncBin = r; }
string SiteInfo::getRsyncBin() const { return rsyncBin; }
void SiteInfo::setPort(const string &p) { setPort(atoi(p.c_str())); }
void SiteInfo::setPort(const int &p) {
	port = (p < 65536) ? p : defaultPort; // is this the right max port #?
}
// TODO: implement some non-modifying length check.. i think..
string SiteInfo::getPort() const {
	// why was this commented out?  caused segfault like that...
	char portStr[PORT_LENGTH + 1];
	//char *portStr;
	sprintf(portStr, "%i", port);
	return portStr;
}
void SiteInfo::setLocalhost(const bool &val) { localhost = val; }
bool SiteInfo::getLocalhost() const { return localhost; }
void SiteInfo::setAnonymous(const bool &val) { anonymous = val; }
bool SiteInfo::getAnonymous() const { return anonymous; }
void SiteInfo::setRootPerms(const bool &val) { useRoot = val; }
bool SiteInfo::getRootPerms() const { return useRoot; }

void SiteInfo::initInfo() {
	setRsyncBin("rsync");
	setHost("");
	setRemote("");
	setLocalItem("");
	setLocalDir("");
	setUser("");
	setPass("");
	setRootPass("");
	setComment("");
	setPort(defaultPort);
	setLocalhost(false);
	setAnonymous(true);
	setRootPerms(false);
}

void SiteInfo::reset() {
	setDescription("No description");
	initInfo();
}
